@@ -8,8 +8,9 @@ package String::Tagged::Terminal::Win32Console;
 use strict;
 use warnings;
 use Win32::Console;
+use List::Util qw( max );
 
-our $VERSION = '0.03_002';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -27,9 +28,20 @@ console on C<MSWin32>. It is not intended to be used directly.
 
 =cut
 
+use constant {
+   ATTR_BLUE          => 0x0001,
+   ATTR_GREEN         => 0x0002,
+   ATTR_RED           => 0x0004,
+   ATTR_INTENSITY     => 0x0008,
+   ATTR_REVERSE_VIDEO => 0x4000, # Windows 10 onwards
+   ATTR_UNDERSCORE    => 0x8000, # Windows 10 onwards
+};
+
 # We can only ever allocate a single console on Windows
 our $WIN32_CONSOLE;
 
+my %color_to_attr; # a cache
+
 sub print_to_console
 {
    my $self = shift;
@@ -44,7 +56,7 @@ sub print_to_console
 
    my %output_options = (
       ( $opts{no_color} ? ( except => [qw( fgindex bgindex )] ) : () ),
-      only => [qw( fgindex bgindex under reverse )], # only process what we can handle
+      only => [qw( fgindex bgindex bold under reverse )], # only process what we can handle
    );
 
    if( $fileno < 0 ) {
@@ -61,22 +73,15 @@ sub print_to_console
 
          # Simple boolean attributes first
          foreach (
-         #   #[ bold      =>  1, 21 ],
-            [ under     =>  0x8000, 0 ],  # COMMON_LVB_UNDERSCORE, Windows 10 onwards
-            # Rendering is flakey under Windows 10
-         #   #[ italic    =>  3, 23 ],
-         #   #[ strike    =>  9, 29 ],
-         #   #[ blink     =>  5, 25 ],
-            [ reverse   =>  0x4000, 0 ], # COMMON_LVB_REVERSE_VIDEO, Windows 10 onwards
+            # bold is handled at the end
+            [ under     =>  ATTR_UNDERSCORE ], # Rendering is flakey under Windows 10
+            # Windows console doesn't support italic, strike, blink
+            [ reverse   =>  ATTR_REVERSE_VIDEO ],
          ) {
-            my ( $tag, $on, $off ) = @$_;
-            if( exists $tags{ $tag } ) {
-               my $keep = 0xFFFF ^ $on;
-               $attr = $attr & $keep;
-               if( $tags{ $tag }) {
-                  $attr = $attr |= $on;
-               }
-            }
+            my ( $tag, $on ) = @$_;
+            $attr &= ~$on;
+
+            $attr |= $on if $tags{$tag};
          }
 
          # Colour index attributes
@@ -85,23 +90,21 @@ sub print_to_console
             [ bgindex => 4, ],
          ) {
             my ( $tag, $shift ) = @$_;
-            if( exists $tags{ $tag }) {
-               my $val = $tags{ $tag };
+            my $mask = 0x000F << $shift;
+            $attr &= ~$mask;
 
-               # The bits for red and blue are swapped between ANSI and Win32 console
-               my $r = $val & 1;
-               my $b = $val & 4;
-               $val &= 0b1111111111111010;
-               $val |= 0b0000000000000100 if( $r );
-               $val |= 0b0000000000000001 if( $b );
-               my $nibble = $val << $shift;
-               my $mask   = 0x000F << $shift;
-               my $keep   = 0xFFFF ^ $mask;
-
-               $attr = $attr & $keep | $nibble;
+            if( defined $tags{$tag} ) {
+               my $idx = $tags{$tag};
+               $attr |= ( $color_to_attr{$idx} //= _color_to_attr( $idx ) ) << $shift;
+            }
+            else {
+               # Restore to previous
+               $attr |= $saved & $mask;
             }
          }
 
+         $attr |= ATTR_INTENSITY if $tags{bold};
+
          $console->Attr($attr);
          $console->Write($s);
       }, %output_options );
@@ -110,9 +113,40 @@ sub print_to_console
    }
 }
 
+sub _color_to_attr
+{
+   my ( $idx ) = @_;
+
+   my $attr = 0;
+
+   if( $idx >= 16 ) {
+      # Attempt to convert xterm256 range into RGB+I
+      require Convert::Color;
+      my $color = Convert::Color->new( "xterm:$idx" )->as_rgb;
+
+      my ( $red, $green, $blue ) = $color->rgb;
+      my $max = max( $red, $green, $blue );
+
+      $attr |= ATTR_RED   if $red   > 0.5;
+      $attr |= ATTR_GREEN if $green > 0.5;
+      $attr |= ATTR_BLUE  if $blue  > 0.5;
+      $attr |= ATTR_INTENSITY if $max > 0.75;
+      $attr = ATTR_INTENSITY if $attr == 0 and
+         $red == $green and $red == $blue and $max > 0.25; # dark grey
+   }
+   else {
+      # The bits are swapped between ANSI and Win32 console
+      $attr |= ATTR_RED   if $idx & 1;
+      $attr |= ATTR_GREEN if $idx & 2;
+      $attr |= ATTR_BLUE  if $idx & 4;
+      $attr |= ATTR_INTENSITY if $idx & 8;
+   }
+   return $attr;
+}
+
 =head1 COMPATIBILITY NOTES
 
-On Windows before Windows 10, only C<fgindex> and C<bgindex> are supported.
+On Windows before Windows 10, only C<fgindex>, C<bgindex> and C<bold> are supported.
 
 Starting with Windows 10, also C<under> and C<reverse> are supported.
 
